(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('@angular/core'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ngx-loading-customizer', ['exports', 'rxjs', 'rxjs/operators', '@angular/core', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-loading-customizer'] = {}, global.rxjs, global.rxjs.operators, global.ng.core, global.ng.common));
}(this, (function (exports, rxjs, operators, core, common) { 'use strict';

    var loadingStates = [
        'idle',
        'loading',
        'loaded',
        'error',
    ];

    function isIdle(loadable) {
        return loadable.type === 'idle';
    }
    function isLoading(loadable) {
        return loadable.type === 'loading';
    }
    function hasErrored(loadable) {
        return loadable.type === 'error';
    }
    function isLoaded(loadable) {
        return loadable.type === 'loaded';
    }
    function isLoadable(object) {
        return (typeof object === 'object' &&
            !!object &&
            'type' in object &&
            loadingStates.includes(object.type));
    }
    function getLoadingState(loadable) {
        return loadable.type;
    }

    var idle = { type: 'idle' };
    var loading = { type: 'loading' };
    function loaded(value) {
        return { type: 'loaded', value: value };
    }
    function errored(error) {
        return { type: 'error', error: error };
    }

    function flatMap(f, loadable) {
        return isLoaded(loadable) ? f(loadable.value) : loadable;
    }

    function fold(constructors, loadable) {
        if (isLoaded(loadable)) {
            return constructors.loaded(loadable.value);
        }
        else if (hasErrored(loadable)) {
            return constructors.error(loadable.error);
        }
        else if (isIdle(loadable)) {
            return constructors.idle();
        }
        else {
            return constructors.loading();
        }
    }

    function fromPromise(promise) {
        return promise.then(loaded).catch(errored);
    }

    function of(value) {
        return loaded(value);
    }

    function map(f, loadable) {
        return flatMap(function (t) { return of(f(t)); }, loadable);
    }

    function toLoadable(observable) {
        return observable.pipe(operators.map(loaded), operators.catchError(function (error) { return rxjs.of(errored(error)); }), operators.startWith(loading));
    }

    var LoadableErrorComponent = /** @class */ (function () {
        function LoadableErrorComponent() {
        }
        return LoadableErrorComponent;
    }());
    LoadableErrorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ld-loadable-error',
                    template: "<h3>Error</h3>\n<pre><code>{{ error | json }}</code></pre>\n",
                    styles: [""]
                },] }
    ];
    LoadableErrorComponent.propDecorators = {
        error: [{ type: core.Input }]
    };

    var LoadableLoadingComponent = /** @class */ (function () {
        function LoadableLoadingComponent() {
        }
        return LoadableLoadingComponent;
    }());
    LoadableLoadingComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ld-loadable-loading',
                    template: "<h3>Loading...</h3>\n",
                    styles: [""]
                },] }
    ];

    var defaultComponents = {
        error: LoadableErrorComponent,
        loading: LoadableLoadingComponent,
    };

    var DEFAULT_COMPONENTS = new core.InjectionToken('DEFAULT_COMPONENTS', {
        factory: function () { return defaultComponents; },
    });

    function setComponentInputs(defaultComponentRef, loadable) {
        defaultComponentRef.instance.type = loadable.type;
        if (isLoaded(loadable)) {
            defaultComponentRef.instance.value = loadable.value;
        }
        else if (hasErrored(loadable)) {
            defaultComponentRef.instance.error = loadable.error;
        }
    }
    function loadableTemplateContext(loadable) {
        return fold({
            idle: function () { return ({}); },
            loading: function () { return ({}); },
            loaded: function (value) { return ({ value: value }); },
            error: function (error) { return ({ error: error }); },
        }, loadable);
    }
    var LoadableComponent = /** @class */ (function () {
        function LoadableComponent(resolver, defaultComponents) {
            this.resolver = resolver;
            this.defaultComponents = defaultComponents;
            this.loadable = idle;
            this.templates = {};
            this.showNgContent = false;
        }
        Object.defineProperty(LoadableComponent.prototype, "loadingState", {
            get: function () {
                return getLoadingState(this.loadable);
            },
            enumerable: false,
            configurable: true
        });
        LoadableComponent.prototype.updateContent = function () {
            if (this.loadingState === 'loaded') {
                return;
            }
            this.templateRef = this.templates[this.loadingState];
            var defaultComponent = this.defaultComponents[this.loadingState];
            if (!this.templateRef && defaultComponent && this.content) {
                this.content.clear();
                var factory = this.resolver.resolveComponentFactory(defaultComponent);
                this.defaultComponentRef = this.content.createComponent(factory);
                setComponentInputs(this.defaultComponentRef, this.loadable);
            }
        };
        LoadableComponent.prototype.ngOnChanges = function (changes) {
            var _a;
            if ('loadable' in changes) {
                this.showNgContent = isLoaded(this.loadable);
                this.templateContext = Object.assign(Object.assign({}, loadableTemplateContext(this.loadable)), { type: this.loadable.type });
            }
            if ('loadable' in changes || 'templates' in changes) {
                (_a = this.defaultComponentRef) === null || _a === void 0 ? void 0 : _a.destroy();
                this.updateContent();
            }
        };
        LoadableComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            setTimeout(function () { return _this.updateContent(); });
        };
        return LoadableComponent;
    }());
    LoadableComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ld-loadable',
                    template: "<ng-content *ngIf=\"showNgContent\"></ng-content>\n<ng-container *ngIf=\"!showNgContent\">\n  <ng-container\n    *ngTemplateOutlet=\"\n      templateRef ? templateRef : content;\n      context: templateContext\n    \"\n  ></ng-container\n></ng-container>\n<ng-template #content></ng-template>\n",
                    styles: [""]
                },] }
    ];
    LoadableComponent.ctorParameters = function () { return [
        { type: core.ComponentFactoryResolver },
        { type: undefined, decorators: [{ type: core.Inject, args: [DEFAULT_COMPONENTS,] }] }
    ]; };
    LoadableComponent.propDecorators = {
        loadable: [{ type: core.Input }],
        templates: [{ type: core.Input }],
        content: [{ type: core.ViewChild, args: ['content', { read: core.ViewContainerRef },] }]
    };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from) {
        for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
            to[j] = from[i];
        return to;
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    var LoadableError = /** @class */ (function (_super) {
        __extends(LoadableError, _super);
        function LoadableError(message) {
            var _this = _super.call(this, message) || this;
            _this.message = message;
            _this.name = 'LoadableError';
            return _this;
        }
        return LoadableError;
    }(Error));

    var IsLoadingPipe = /** @class */ (function () {
        function IsLoadingPipe() {
        }
        IsLoadingPipe.prototype.transform = function (object) {
            if (!isLoadable(object)) {
                throw new LoadableError('The passed object is not a loadable');
            }
            return isLoading(object);
        };
        return IsLoadingPipe;
    }());
    IsLoadingPipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'isLoading',
                },] }
    ];

    var LoadedValuePipe = /** @class */ (function () {
        function LoadedValuePipe() {
        }
        LoadedValuePipe.prototype.transform = function (object) {
            if (!isLoadable(object)) {
                throw new LoadableError('The passed object is not a loadable');
            }
            return isLoaded(object) ? object.value : undefined;
        };
        return LoadedValuePipe;
    }());
    LoadedValuePipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'loadedValue',
                },] }
    ];

    var LoadingStatePipe = /** @class */ (function () {
        function LoadingStatePipe() {
        }
        LoadingStatePipe.prototype.transform = function (object) {
            if (!isLoadable(object)) {
                throw new LoadableError('The passed object is not a loadable');
            }
            return getLoadingState(object);
        };
        return LoadingStatePipe;
    }());
    LoadingStatePipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'loadingState',
                },] }
    ];

    var LoadingCustomizerModule = /** @class */ (function () {
        function LoadingCustomizerModule() {
        }
        LoadingCustomizerModule.forRoot = function (options) {
            if (options === void 0) { options = {}; }
            return {
                ngModule: LoadingCustomizerModule,
                providers: [
                    {
                        provide: DEFAULT_COMPONENTS,
                        useValue: Object.assign(Object.assign({}, defaultComponents), options.defaultComponents),
                    },
                ],
            };
        };
        return LoadingCustomizerModule;
    }());
    LoadingCustomizerModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [
                        IsLoadingPipe,
                        LoadedValuePipe,
                        LoadingStatePipe,
                        LoadableComponent,
                        LoadableErrorComponent,
                        LoadableLoadingComponent,
                    ],
                    imports: [common.CommonModule],
                    exports: [
                        IsLoadingPipe,
                        LoadedValuePipe,
                        LoadingStatePipe,
                        LoadableComponent,
                        LoadableErrorComponent,
                        LoadableLoadingComponent,
                    ],
                    entryComponents: [LoadableLoadingComponent, LoadableErrorComponent],
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.DEFAULT_COMPONENTS = DEFAULT_COMPONENTS;
    exports.IsLoadingPipe = IsLoadingPipe;
    exports.LoadableComponent = LoadableComponent;
    exports.LoadableError = LoadableError;
    exports.LoadableErrorComponent = LoadableErrorComponent;
    exports.LoadableLoadingComponent = LoadableLoadingComponent;
    exports.LoadedValuePipe = LoadedValuePipe;
    exports.LoadingCustomizerModule = LoadingCustomizerModule;
    exports.LoadingStatePipe = LoadingStatePipe;
    exports.defaultComponents = defaultComponents;
    exports.errored = errored;
    exports.flatMap = flatMap;
    exports.fold = fold;
    exports.fromPromise = fromPromise;
    exports.getLoadingState = getLoadingState;
    exports.hasErrored = hasErrored;
    exports.idle = idle;
    exports.isIdle = isIdle;
    exports.isLoadable = isLoadable;
    exports.isLoaded = isLoaded;
    exports.isLoading = isLoading;
    exports.loaded = loaded;
    exports.loading = loading;
    exports.loadingStates = loadingStates;
    exports.map = map;
    exports.of = of;
    exports.setComponentInputs = setComponentInputs;
    exports.toLoadable = toLoadable;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-loading-customizer.umd.js.map
